#pragma rtGlobals = 1		// modern global method

#include <Strings as Lists>
#include <Keyword-Value>

// PPT Peak Analysis - control panel for peak analysis
// part of the PPT, the "Patcher's Power Tools"
//  25.11.96 Francisco Mendez

// 01.12.96 panel is positioned next to the top graph
// 02.12.96 range can be point- or x-range (for "Minis-Detection")
// 04.12.96 new buttons to browse through the found peaks
// 04.12.96 graph will be scrolled in x-direction if cursor gets out of range
// 06.12.96 graph will be scrolled also in y-direction
// 12.01.97 SetVariables are no more "Chicago" (problems with system font replacement)
// 23.10.97 macro stores PPT-globals in folder "PPTGlobals", Igor Globals are not created
// 27.10.97 procedures changed into functions

// ****************************************************

Menu "Macros"
	"Peak Analysis Panel", PPTPeakAnalysis ()
End

// ****************************************************

Function PPTPeakGlobals ()
	Silent 1

	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals	// create the PPT global variables

	Variable /G		pptMinAmp			// peak threshold - set variables
	Variable /G		pptBoxNum			// number for sliding average
	Variable /G		pptPeakX = NaN		// x-Value of actual Peak - display values
	Variable /G		pptPeakY = NaN		// y-Value of actual Peak
	Variable /G		pptHasMinMax = 0	// minimum/maximum of wave calculated
	Variable /G		pptWaveMin		// min value of actual wave
	Variable /G		pptWaveMax		// max value of current wave
	String /G		pptPeakWave		// actual wave to be analyzed

	SetDataFolder $oldDataFolder
End Function // Function

// ****************************************************

Function PPTPeakAnalysis () : Panel
	// PauseUpdate; Silent 1			// building window...

	Variable	panelX
	variable	panelY
	Variable	panelW = 434
	Variable	panelH = 120

	if (strlen (WinName (0, 1)) <= 0)
		Abort "Please create a graph before doing any peak analysis!"
	endif

	GetWindow kwTopWin, wsize	// place control panel to the right of the top graph
	panelX = V_right
	panelY = V_top

	PPTPeakGlobals ()
	NewPanel /W=(panelX, panelY, panelX + panelW, panelY + panelH) as "PPT Peak Analysis"
	Execute "ModifyPanel cbRGB = (61166, 61166, 61166)"		// grey background

	PopupMenu pYWave	pos = {4, 2}, size = {122, 19}
	PopupMenu pYWave	title = "Wave"
	PopupMenu pYWave	mode = 1
	PopupMenu	pYWave	value = PPTGetYWaveList ()
	PopupMenu	pYWave	proc = PPTChangeYWave
	PopupMenu pYWave	help = {"Selects a wave for peak analysis.\r\rIf cursor A is placed on another wave, this wave will be used instead."}

	CheckBox cFindMinima	pos = {4, 22}, size = {100, 20}, title = "Find Minima"
	CheckBox cFindMinima	value = 0
	CheckBox cFindMinima	help = {"If cecked, wave minima will be searched, and threshold is the maximal amplitude needed. Otherwise maxima above threshold will be searched."}

	CheckBox cPointRange	pos = {4, 42}, size = {100, 20}, title = "Point Range"
	CheckBox cPointRange	value = 0
	CheckBox cPointRange	help = {"If checked, the peak location will be found as point number (index), otherwise as x-value.\r\rIf you encounter difficulties in finding any peaks, uncheck this option."}

	Button bAutoPeaks	pos = {4, 68}, size = {124, 20}, title = "Auto Detection"
	Button bAutoPeaks	proc = PPTAutoPeaks
	Button bAutoPeaks	help = {"Finds peaks of all displayed waves and adds them to the peak tables."}

	Button bFindPeak	pos = {4, 94}, size = {124, 20}, title = "Find Next Peak"
	Button bFindPeak	proc = PPTFindNextPeak
	Button bFindPeak	help = {"Finds next peak in the current wave by analyzing the smoothed first and second derivative. Beeps if none was found."}

	SetVariable vBox	pos = {160, 2}, size = {130, 17}, title = "Smoothing"
	SetVariable vBox	format ="%d", limits = {1, INF, 1}
	SetVariable vBox	value = root:PPTGlobals:pptBoxNum
	SetVariable vBox	help = {"Sets the box for sliding average."}

	SetVariable	vMinAmp	pos = {160, 22}, size = {130, 17}, title = "Threshold"
	SetVariable	vMinAmp	limits = {-INF, INF, 0}
	SetVariable	vMinAmp	value = root:PPTGlobals:pptMinAmp
	SetVariable	vMinAmp	help = {"Defines minimum level of a peak (maximum level if minima should be found)."}

	Button bPreviousPeak	pos = {160, 42}, size = {124, 20}, title = "Previous Peak"
	Button bPreviousPeak	proc = PPTShowPeak
	Button bPreviousPeak	help = {"Sets the cursor to the previous peak in the peaks-table. Beeps if there are no more entries."}

	Button bDeletePeak	pos = {160, 68}, size = {124, 20}, title = "Delete from Table"
	Button bDeletePeak	proc = PPTDeletePeakFromTable
	Button bDeletePeak	help = {"Deletes the current value of cursor A from the peaks tables. Beeps if the value is not there."}

	Button bAddPeak	pos = {160, 94}, size = {124, 20}, title = "Add to Table"
	Button bAddPeak	proc = PPTAddPeakToTable
	Button bAddPeak	help = {"Adds the current value of cursor A to the peaks tables. Beeps if the value is there already."}

	ValDisplay dPeakY	pos = {300, 2}, size = {130, 17}, title = "Amplitude"
	ValDisplay dPeakY	limits = {0, 0, 0}, barmisc = {0, 1000}
	ValDisplay dPeakY	help = {"Displays current peak amplitude."}

	ValDisplay dPeakX	pos = {300, 22}, size = {130, 17}, title = "X-Position"
	ValDisplay dPeakX	limits = {0, 0, 0}, barmisc = {0, 1000}
	ValDisplay dPeakX	help = {"Displays current peak location."}

	Button bNextPeak	pos = {306, 42}, size = {124, 20}, title = "Next Peak"
	Button bNextPeak	proc = PPTShowPeak
	Button bNextPeak	help = {"Sets the cursor to the next peak in the peaks-table. Beeps if there are no more entries."}

	Button bGetThreshold	pos = {306, 68}, size = {124, 20}, title = "Get Threshold"
	Button bGetThreshold	proc = PPTGetThreshold
	Button bGetThreshold	help = {"Reads back current cursor position into the minimum amplitude level."}

	Button bOpenTable	pos = {306, 94}, size = {124, 20}, title = "Open Peaks Table"
	Button bOpenTable	proc = PPTOpenPeakTable
	Button bOpenTable	help = {"Opens a table with current peak locations and amplitudes. The waves are created automatically if they don't exist."}

	Execute "PPTSetPeakDisplayValues ()"

	// ResumeUpdate
End Window // PPTPeakAnalysis

// ****************************************************

Proc PPTSetPeakDisplayValues ()		// Doesn't work within functions
	ValDisplay dPeakY value = root:PPTGlobals:pptPeakY
	ValDisplay dPeakX value = root:PPTGlobals:pptPeakX
End Proc		// PPTSetPeakDisplayValues

// ****************************************************
// create names for location- and amplitude-waves from source wavename 
// ****************************************************

Function /S PPTMakeLocsWaveName (theWaveName)
	String theWaveName

	if (strlen (theWaveName) > 0)
		return (theWaveName + "_locs")
	else
		return ("")
	endif
End Function // PPTMakeLocsWaveName

// ****************************************************

Function /S PPTMakePeaksWaveName (theWaveName)
	String theWaveName

	if (strlen (theWaveName) > 0)
		return (theWaveName + "_peaks")
	else
		return ("")
	endif
End Function // PPTMakePeaksWaveName

// ****************************************************
// sets the cursor at the specified position, scrolls grah if necessary
// ****************************************************

Proc PPTSetCursor (cursorName, waveName, position, pointMode)
	String		cursorName = "A"
	String		waveName
	Variable	position = 0
	Variable	pointMode = 0
	prompt cursorName "Cursor to set", popup "A;B"
	prompt waveName "Wave in target graph", popup PPTGetYWaveList ()
	prompt position "Cursor position"
	prompt pointMode "Mode", popup "Position is X-Value;Position is point number"

	Silent 1; PauseUpdate

	if (strlen (waveName) <= 0)
		Abort "Please specify a wave to place the cursor in!"
	endif
	if (strlen (cursorName) <= 0)
		cursorName = "A"
	endif

	PPTSetCursorAndScroll (cursorName, waveName, position, pointMode)

	ResumeUpdate; Silent 0
End Proc // PPTSetCursor

// ****************************************************

Function PPTSetCursorAndScroll (cursorName, yWaveName, position, pointMode)
	String		cursorName
	String		yWaveName
	Variable	position
	Variable	pointMode

	Variable	leftVal, rightVal, bottomVal, topVal
	Variable	leftDisplay, rightDisplay, bottomDisplay, topDisplay
	String		xAxis, yAxis, theTraceInfo, xWaveName
	Variable	cursorX, cursorY
	Variable	V_min, V_max

	if (pointMode == 1)
		Cursor /P $cursorName $yWaveName position
	else
		Cursor $cursorName $yWaveName position
	endif 

	WAVE yWave = $yWaveName
	theTraceInfo = TraceInfo ("", yWaveName, 0)
	xAxis = StrByKey ("XAXIS", theTraceInfo)
	yAxis = StrByKey ("YAXIS", theTraceInfo)
	xWaveName = StrByKey ("XWAVE", theTraceInfo)
	WAVE xWave = $xWaveName

	GetAxis /Q $xAxis						// get axis-range
	leftDisplay = V_min
	rightDisplay = V_max
	leftVal = yWave [0]
	if (WaveExists (xWave))
		leftVal = xWave [0]
		rightVal = xWave [numpnts (xWave) - 1]
	else
		leftVal = pnt2x (yWave, yWave [0])
		rightVal = pnt2x (yWave, numpnts (yWave) - 1)
	endif

	GetAxis /Q $yAxis
	bottomDisplay = V_min
	topDisplay = V_max

	NVAR	hasMinMax	= root:PPTGlobals:pptHasMinMax
	NVAR	waveMin	= root:PPTGlobals:pptWaveMin
	NVAR	waveMax	= root:PPTGlobals:pptWaveMax
	SVAR	peakWave	= root:PPTGlobals:pptPeakWave

	if ((hasMinMax == 0) %| (cmpstr (yWaveName, peakWave)))
		WaveStats /Q yWave
		waveMin = V_min
		waveMax = V_max

		hasMinMax = 1
		peakWave = yWaveName
	endif
	bottomVal = waveMin
	topVal = waveMax

	// check Cursor-Range and scroll axes if necessary
	cursorX = hcsr ($cursorName)
	cursorY = vcsr ($cursorName)
	if (cursorX < leftDisplay)
		PPTScrollAxis (xAxis, leftDisplay, rightDisplay, cursorX, leftVal, rightVal, 0)
	endif
	if (cursorX > rightDisplay)
		PPTScrollAxis (xAxis, leftDisplay, rightDisplay, cursorX, leftVal, rightVal, 1)
	endif
	if (cursorY < bottomDisplay)
		PPTScrollAxis (yAxis, bottomDisplay, topDisplay, cursorY, bottomVal, topVal, 0)
	endif
	if (cursorY > topDisplay)
		PPTScrollAxis (yAxis, bottomDisplay, topDisplay, cursorY, bottomVal, topVal, 1)
	endif

	DoUpdate
End Proc // PPTSetCursorAndScroll

// ****************************************************

Function PPTScrollAxis (theAxis, lower, upper, value, min, max, up_right)
	String		theAxis			// axis to scroll
	Variable	lower, upper	// boundaries of the axis
	Variable	value			// this value should be within the axis boundaries
	Variable	min, max		// extents of the wave to display
	Variable	up_right		// true =: scroll up/right, false =: scroll left/down

	Variable	delta = (upper - lower) / 2
	Variable	newLower = lower
	Variable	newUpper = upper

	if (up_right == 1)			// scroll up or right
		do
			newLower += delta
			newUpper += delta
		while (newUpper < value)
		if (newUpper > max)
			newUpper = max
			newLower = max - 2 * delta
		endif
	else							// scroll left or down
		do
			newLower -= delta
			newUpper -= delta
		while (newLower > value)
		if (newLower < min)
			newLower = min
			newUpper = min + 2 * delta
		endif
	endif

	SetAxis /Z $theAxis, newLower, newUpper
End Function // PPTScrollAxis

// ****************************************************
// Create wavelist of y-Waves
// ****************************************************

Function /S PPTGetYWaveList ()
	String		theXWaveList, theYWaveList
	String		tempWaveList
	String		theWaveName
	Variable	i

	theXWaveList = XWaveName ("", "")
	if (strlen (theXWaveList) <= 0)
		theXWaveList = " "
	endif

	tempWaveList = WaveList ("*", ";", "WIN:" + WinName (0, 1))

	i = 0
	theYWaveList = ""
	do
		theWaveName = GetStrFromList (tempWaveList, i, ";")
		if (strlen (theWaveName) == 0)					//end of list found
			break
		endif
		if (strsearch (theWaveName, theXWaveList, 0) < 0)	// not an x-wave...
			theYWaveList += theWaveName + ";"			// ...add to the list
		endif
		i += 1
	while (1)

	return (theYWaveList)
End Function // PPTGetYWaveList

// ****************************************************
// changes the cursor wave if user selects a new wave from the popup
// ****************************************************

Function PPTChangeYWave (ctrlName, popNum, popStr) : PopupMenuControl
	String		ctrlName
	Variable	popNum
	String		popStr

	Variable	V_PeakLoc
	Variable	V_Flag
	String		S_Value
	String		theWaveName
	NVAR		hasMinMax = root:PPTGlobals:pptHasMinMax

	ControlInfo pYWave
	theWaveName = S_Value

	V_PeakLoc = 0
	V_Flag = 1
	hasMinMax = 0

	PPTSetCursorAndScroll ("A", theWaveName, 0, 1)
End Function // PPTChangeYWave

// ****************************************************
// opens a table containing the peak waves of the top graph
// ****************************************************

Function PPTOpenPeakTable (ctrlName) : ButtonControl
	String		ctrlName

	Variable	i
	String		theWaveName, theWaveList
	String		tableLocName
	String		tablePeakName

	theWaveList = PPTGetYWaveList ()
	if (strlen (theWaveList) <= 0)
		Abort "The target window does not contain any waves! Please add at least one wave!"
	endif

	GetWindow kwTopWin, wsize
	Edit /W=(V_left, V_bottom + 20, V_right, V_bottom + 240)

	i = 0
	do
		theWaveName = GetStrFromList (theWaveList, i, ";")
		if (strlen (theWaveName) == 0)					//end of list found
			break
		endif
		tableLocName = PPTMakeLocsWaveName (theWaveName)
		tablePeakName = PPTMakePeaksWaveName (theWaveName)

		PPTCheckPeakWave (tableLocName)
		PPTCheckPeakWave (tablePeakName)
		AppendToTable $tableLocName, $tablePeakName

		i += 1
	while (1)
End Function // PPTOpenPeakTable

// ****************************************************
// check if a given wave exists in the top graph
// ****************************************************

Function PPTWaveIsInTopGraph (theWaveName)
	String	theWaveName

	String	theWaveList

	theWaveList = WaveList ("*", ";", "WIN:" + WinName (0, 1))
	return (strsearch (theWaveList, theWaveName, 0))
End Function // PPTWaveIsInTopGraph

// ****************************************************
// find next peak in the target wave
// ****************************************************

Function PPTFindNextPeak (ctrlName) : ButtonControl
	String ctrlName

	String		xWaveName
	String		yWaveName
	Variable	findMinima, pointRange
	String		tempStr

	Variable	V_Value
	String		S_Value

	NVAR	boxNum		= root:PPTGlobals:pptBoxNum
	NVAR	minAmp	= root:PPTGlobals:pptMinAmp
	NVAR	peakX		= root:PPTGlobals:pptPeakX
	NVAR	peakY		= root:PPTGlobals:pptPeakY
	NVAR	peakLoc		= root:V_PeakLoc		// will be created by FindPeaks anyway :-((
	NVAR	flag			= root:V_Flag

	ControlInfo cFindMinima
	findMinima = V_Value

	ControlInfo cPointRange
	pointRange = V_Value

	yWaveName = CsrWave (A)
	if (strlen (yWaveName) <= 0)
		ControlInfo pYWave
		yWaveName = S_Value
		if (PPTWaveIsInTopGraph (yWaveName) < 0)
			Abort "Please specify a wave by setting Cursor A or by selecting an item from the wave popup!"
		endif
		PPTSetCursorAndScroll ("A", yWaveName, 0, 1)
	endif

	if (boxNum < 1)			// make sure to have a valid sliding av. number
		boxNum = 1				// minimum allowed
	endif

	// build up command line
	String theCommand
	sprintf theCommand "FindPeak /Q/B=%d/M=%g", boxNum, minAmp
	if (pointRange)
		sprintf tempStr, "/P/R=[%d]", xcsr (A) + 1
	else
		sprintf tempStr, "/R=(%g)", hcsr (A) + deltax ($yWaveName)
	endif
	theCommand += tempStr
	if (findMinima)
		theCommand += "/N"
	endif
	theCommand += (" " + yWaveName)
	Execute theCommand

	if (flag == 1)								// No peak was found
		Beep
		return 0
	else
		if (pointRange)
			PPTSetCursorAndScroll ("A", yWaveName, peakLoc, 1)
		else
			PPTSetCursorAndScroll ("A", yWaveName, peakLoc, 0)
		endif
		peakX = hcsr (A)
		peakY = vcsr (A)
		
		return 1
	endif
End Function // PPTFindNextPeak

// ****************************************************
// places the cursor to the peak before/after current cursor position
// ****************************************************

Function PPTShowPeak (ctrlName) : ButtonControl
	String		ctrlName

	Variable	newCursorLoc, oldCursorLoc
	String		waveName, locsWaveName
	NVAR 		peakX	= root:PPTGlobals:pptPeakX
	NVAR 		peakY	= root:PPTGlobals:pptPeakY

	waveName = CsrWave (A)
	if (strlen (waveName) <= 0)
		Abort "Please drag cursor A to the target wave!"
	else
		locsWaveName = PPTMakeLocsWaveName (waveName)
	endif

	oldCursorLoc = hcsr (A)

	if (cmpstr (ctrlName, "bPreviousPeak") == 0)
		newCursorLoc = PPTGetPrevPeak ($locsWaveName, oldCursorLoc)
	else
		newCursorLoc = PPTGetNextPeak ($locsWaveName, oldCursorLoc)
	endif

	if (newCursorLoc >= 0)
		PPTSetCursorAndScroll ("A", waveName, newCursorLoc, 0)
		peakX = hcsr (A)
		peakY = vcsr (A)
	else
		Beep
	endif
End Function // PPTShowPeak

// ****************************************************

Function PPTGetNextPeak (theWave, value)
	Wave		theWave
	Variable	value

	Variable	i, num

	num = numpnts (theWave)
	if (num <= 0)
		return (-1)
	endif

	i = 0
	do
		if (cmpnum (theWave [i], value, 10) > 0)
			return (theWave [i])
		endif
		i += 1
	while (i < num)

	return (-1)
End Function // PPTGetNextPeak

// ****************************************************
// Returns index of previous peak in peaks table
// ****************************************************

Function PPTGetPrevPeak (theWave, theValue)
	Wave		theWave
	Variable	theValue

	Variable	i, num

	num = numpnts (theWave)
	if (num <= 0)
		return (-1)
	endif

	i = num - 1
	do
		if (cmpnum (theWave [i], theValue, 10) < 0)
			return (theWave [i])
		endif
		i -= 1
	while (i >= 0)

	return (-1)
End Function				// PPTGetPrevPeak

// ****************************************************
// adds actual peak (value at cursor A) to the corresponding wave
// returns FALSE if value was in table, TRUE else
// ****************************************************

Function PPTAddPeakToTable (ctrlName) : ButtonControl
	String		ctrlName

	Variable	index
	String		tableLocName
	String		tablePeakName
	Variable	peakXVal, peakYVal
	String		thePeakWave

	thePeakWave = CsrWave (A)
	if (strlen (thePeakWave) <= 0)
		Abort "This adds the current cursor-position to the peaks-table. Since cursor A is undefined no value has been added!"
	else
		tableLocName = PPTMakeLocsWaveName (thePeakWave)
		tablePeakName = PPTMakePeaksWaveName (thePeakWave)
	endif

	PPTCheckPeakWave (tableLocName)
	PPTCheckPeakWave (tablePeakName)

	Wave	tableLocWave = $tableLocName
	Wave	tablePeakWave = $tablePeakName

	peakYVal = vcsr (A)
	peakXVal = hcsr (A)
	index = PPTGetPeakIndex ($tableLocName, peakXVal)

	if (index >= 0)
		InsertPoints (index), 1, tableLocWave
		tableLocWave [index] = peakXVal
		InsertPoints (index), 1, tablePeakWave
		tablePeakWave [index] = peakYVal
		return 1		// value inserted
	else
		Beep			// value not inserted
		return 0
	endif
End Function			// PPTAddPeakToTable

// ****************************************************
// get position where peak has to be inserted into a wave, -1 if peak is already there
// ****************************************************

Function PPTGetPeakIndex (theWave, theValue)
	Wave		theWave
	Variable	theValue

	Variable	i
	Variable	num

	num = numpnts (theWave)
	if (num <= 0)
		return (0)			// no values in table, add into start position
	endif

	i = 0
	do
		if (theValue < theWave [i])
			break
		endif
		i += 1
	while (i < num)

	if ((cmpnum (theValue, theWave [i - 1], 10) == 0) %| (cmpnum (theValue, theWave [i], 10) == 0))
		return (-1)		// peak already in the table
	else
		return (i)			// peak not in table, return position
	endif
End Function				// PPTGetPeakIndex

// ****************************************************
// Check if wave for peakvalues exists and create one if not the case
// ****************************************************

Function PPTCheckPeakWave (waveName)
	String	waveName

	if (! WaveExists ($waveName))
		Make /N=0 $waveName		// create empty wave
	endif
End Function						// PPTCheckPeakWave

// ****************************************************
// deletes the actual peak from the peak-tables
// returns TRUE if value was in table, FALSE else
// ****************************************************

Function PPTDeletePeakFromTable (ctrlName) : ButtonControl
	String		ctrlName

	Variable	index
	Variable	peakXVal
	String		thePeakWave
	String		tableLocName, tablePeakName

	thePeakWave = CsrWave (A)
	if (strlen (thePeakWave) <= 0)
		Abort "This deletes the current cursor-position from the peaks-table. Since cursor A is undefined no value has been deleted!"
	else
		tableLocName = PPTMakeLocsWaveName (thePeakWave)
		tablePeakName = PPTMakePeaksWaveName (thePeakWave)
	endif

	WAVE	tableLocWave = $tableLocName
	WAVE	tablePeakWave = $tablePeakName

	peakXVal = hcsr (A)
	index = BinarySearch (tableLocWave, peakXVal)	// find value in peaks table
	if (index == -2)
		index = numpnts (tableLocWave) - 1
	endif
	if (index >= 0)
		if (cmpnum (peakXVal, tableLocWave [index], 10) == 0)
			DeletePoints (index), 1, tableLocWave
			DeletePoints (index), 1, tablePeakWave
			return 1
		endif
	endif
	if (index < numpnts (tableLocWave))
		if (cmpnum (peakXVal, tableLocWave [index + 1], 10) == 0)
			DeletePoints (index + 1), 1, tableLocWave
			DeletePoints (index + 1), 1, tablePeakWave
			return 1
		endif
	endif

	Beep			// peak was not in the table
	return 0		// FALSE
End Function		// PPTDeletePeakFromTable

// ****************************************************
// reads back threshold value according to the actual cursor position
// ****************************************************

Function PPTGetThreshold (ctrlName) : ButtonControl
	String	ctrlName
	String	yWaveName

	NVAR	minAmp = root:PPTGlobals:pptMinAmp

	yWaveName = CsrWave (A)
	if (strlen (yWaveName) <= 0)
		Abort "This defines the current cursor-position as new threshold. Since cursor A is undefined threshold remains unchanged!"
	else
		minAmp = vcsr (A)
	endif
End Function // PPTGetThreshold

// ****************************************************
// steps automatically through all waves of the target window, finds all peaks
// there and adds them to the peak table
// ****************************************************

Function PPTAutoPeaks (ctrlName) : ButtonControl
	String		ctrlName

	String		theWaveList
	String		theWaveName
	Variable	i = 0
	Variable	waveNum = 0
	Variable	peakNum = 0
	String		alertStr

	String topGraph = WinName (0, 1)			// get name of top graph...
	DoWindow /F $topGraph						// ...and bring it to the front, so it's visible
	String panelName = WinName (0, 64)		// now bring panel to front
	DoWindow /F $panelName

	theWaveList = PPTGetYWaveList ()
	if (strlen (theWaveList) <= 0)
		Abort "There are no y-waves in the target window, so no peaks can be detected!"
	endif

	do
		theWaveName = GetStrFromList (theWaveList, i, ";")
		if (strlen (theWaveName) == 0)		//end of list found, exit loop
			break
		endif

		waveNum += 1
		PopupMenu pYWave mode = (i + 1)		// change y-Wave Popupvalue
		PPTChangeYWave ("", 0, "")

		do
			if (PPTFindNextPeak ("") == 0)
				break
			endif
			Execute "PPTAddPeakToTable (\"\")"
			peakNum += 1
		while (1)

		i += 1
	while (1)
	
	sprintf alertStr, "%d peaks found in %d waves!", peakNum, waveNum
	DoAlert 0, alertStr
End Function // PPTAutoPeaks
